@@ -19,6 +19,14 @@ class NoRepo(GTOException):
         super().__init__(self.message)
 
 
+class WrongConfig(GTOException):
+    _message = "Wrong config file '{path}'"
+
+    def __init__(self, path) -> None:
+        self.message = self._message.format(path=path)
+        super().__init__(self.message)
+
+
 class NoFile(GTOException):
     _message = "No file/folder found in '{path}' for checkouted commit"
 
@@ -28,9 +36,7 @@ class NoFile(GTOException):
 
 
 class UnknownType(GTOException):
-    _message = (
-        "Type '{type}' is not present in your config file. Allowed values are: {types}."
-    )
+    _message = "Allowed types are: '{types}'"
 
     def __init__(self, type, types) -> None:
         self.message = self._message.format(type=type, types=types)
@@ -38,7 +44,7 @@ class UnknownType(GTOException):
 
 
 class ArtifactExists(GTOException):
-    _message = "Artifact '{name}' is already exists in Index"
+    _message = "Enrichment for '{name}' already exists"
 
     def __init__(self, name) -> None:
         self.message = self._message.format(name=name)
@@ -46,7 +52,7 @@ class ArtifactExists(GTOException):
 
 
 class ArtifactNotFound(GTOException):
-    _message = "Artifact '{name}' doesn't exist in Index"
+    _message = "Cannot find artifact '{name}'"
 
     def __init__(self, name) -> None:
         self.message = self._message.format(name=name)
@@ -54,7 +60,7 @@ class ArtifactNotFound(GTOException):
 
 
 class PathIsUsed(GTOException):
-    _message = "Provided path conflicts with {path} ({type} {name})"
+    _message = "Provided path conflicts with '{path}' ('{type}' '{name}')"
 
     def __init__(self, type, name, path) -> None:
         self.message = self._message.format(type=type, name=name, path=path)
@@ -70,7 +76,7 @@ class VersionRequired(GTOException):
 
 
 class ManyVersions(GTOException):
-    _message = "{versions} versions of artifact {name} found"
+    _message = "'{versions}' versions of artifact '{name}' found"
 
     def __init__(self, name, versions) -> None:
         self.message = self._message.format(name=name, versions=versions)
@@ -86,23 +92,15 @@ class VersionAlreadyRegistered(GTOException):
 
 
 class VersionExistsForCommit(GTOException):
-    _message = "The model {model} was already registered in this commit with version '{version}'."
+    _message = "'{model}' is already registered in this commit with version '{version}'"
 
     def __init__(self, model, version) -> None:
         self.message = self._message.format(model=model, version=version)
         super().__init__(self.message)
 
 
-class VersionIsOld(GTOException):
-    _message = "Version '{suggested}' is younger than the latest {latest}"
-
-    def __init__(self, latest, suggested) -> None:
-        self.message = self._message.format(latest=latest, suggested=suggested)
-        super().__init__(self.message)
-
-
 class UnknownStage(GTOException):
-    _message = "Stage '{stage}' is not present in your config file. Allowed stages are: {stages}."
+    _message = "Allowed stages are: '{stages}'"
 
     def __init__(self, stage, stages) -> None:
         self.message = self._message.format(stage=stage, stages=stages)
@@ -126,15 +124,11 @@ class RefNotFound(GTOException):
 
 
 class InvalidVersion(GTOException):
-    _message = "Supplied version {version} doesn't look like {cls} version"
-
-    def __init__(self, version, cls) -> None:
-        self.message = self._message.format(version=version, cls=cls)
-        super().__init__(self.message)
+    pass
 
 
 class IncomparableVersions(GTOException):
-    _message = "You can compare only versions of the same system, but not {} and {}"
+    _message = "You can compare only versions of the same system, but not '{}' and '{}'"
 
     def __init__(self, this, that) -> None:
         self.message = self._message.format(this, that)
@@ -142,7 +136,7 @@ class IncomparableVersions(GTOException):
 
 
 class UnknownAction(GTOException):
-    message = "Unknown action '{action}' was requested."
+    message = "Unknown action '{action}' was requested"
 
     def __init__(self, action) -> None:
         self.message = self.message.format(action=action)
@@ -150,8 +144,32 @@ class UnknownAction(GTOException):
 
 
 class MissingArg(GTOException):
-    message = "'{arg}' is required."
+    message = "'{arg}' is required"
 
     def __init__(self, arg) -> None:
         self.message = self.message.format(arg=arg)
         super().__init__(self.message)
+
+
+class WrongArgs(GTOException):
+    pass
+
+
+class InvalidTagName(GTOException):
+    message = "Cannot parse tag name '{tag}'"
+
+    def __init__(self, tag) -> None:
+        self.message = self.message.format(tag=tag)
+        super().__init__(self.message)
+
+
+class TagExists(GTOException):
+    message = "tag '{name}' already exists"
+
+    def __init__(self, name) -> None:
+        self.message = self.message.format(name=name)
+        super().__init__(self.message)
+
+
+class ValidationError(GTOException):
+    pass
