@@ -1,11 +1,13 @@
 import logging
 from datetime import datetime
+from enum import Enum
 from typing import FrozenSet, Iterable, Optional, Union
 
 import git
 from pydantic import BaseModel
 
-from gto.index import Artifact
+from gto.config import assert_name_is_valid, check_name_is_valid
+from gto.versions import SemVer
 
 from .base import (
     BaseArtifact,
@@ -14,8 +16,15 @@ from .base import (
     BaseRegistryState,
     BaseVersion,
 )
-from .constants import ACTION, NAME, NUMBER, STAGE, VERSION, Action
-from .exceptions import MissingArg, RefNotFound, UnknownAction
+from .constants import ACTION, NAME, NUMBER, STAGE, TAG, VERSION, Action
+from .exceptions import (
+    InvalidTagName,
+    InvalidVersion,
+    MissingArg,
+    RefNotFound,
+    TagExists,
+    UnknownAction,
+)
 
 ActionSign = {
     Action.REGISTER: "@",
@@ -29,53 +38,107 @@ def name_tag(
     version: Optional[str] = None,
     stage: Optional[str] = None,
     repo: Optional[git.Repo] = None,
+    simple: bool = False,
 ):
     if action == Action.REGISTER:
         return f"{name}{ActionSign[action]}{version}"
 
     if action == Action.PROMOTE:
+        if simple:
+            return f"{name}{ActionSign[action]}{stage}"
         if repo is None:
             raise MissingArg(arg="repo")
         numbers = []
         for tag in repo.tags:
             parsed = parse_name(tag.name)
-            if parsed[ACTION] in (Action.PROMOTE,):
+            if (
+                (parsed[NAME] == name)
+                and (parsed[ACTION] == Action.PROMOTE)
+                and (NUMBER in parsed)
+            ):
                 numbers.append(parsed[NUMBER])
         new_number = max(numbers) + 1 if numbers else 1
-        return f"{name}{ActionSign[action]}{stage}-{new_number}"
+        return f"{name}{ActionSign[action]}{stage}{ActionSign[action]}{new_number}"
     raise UnknownAction(action=action)
 
 
-def parse_name(name: str, raise_on_fail: bool = True):
-
-    if ActionSign[Action.REGISTER] in name:
-        name, version = name.split(ActionSign[Action.REGISTER])
+def _parse_register(name: str, raise_on_fail: bool = True):
+    sign = len(name) - name[::-1].index(ActionSign[Action.REGISTER])
+    name, version = name[: sign - 1], name[sign:]
+    if check_name_is_valid(name) and SemVer.is_valid(version):
         return {
             ACTION: Action.REGISTER,
             NAME: name,
             VERSION: version,
         }
+    if raise_on_fail:
+        assert_name_is_valid(name)
+        if not SemVer.is_valid(version):
+            raise InvalidVersion(
+                f"Version format should be v1.0.0, cannot parse {version}"
+            )
+    return {}
+
+
+def _parse_promote(name: str, raise_on_fail: bool = True):
+    parsed = name.split(ActionSign[Action.PROMOTE])
+    if (
+        check_name_is_valid(parsed[0])
+        and check_name_is_valid(parsed[1])
+        and (parsed[2].isdigit() if len(parsed) == 3 else True)
+    ):
+        if len(parsed) == 2:
+            return {
+                ACTION: Action.PROMOTE,
+                NAME: parsed[0],
+                STAGE: parsed[1],
+            }
+        if len(parsed) == 3:
+            return {
+                ACTION: Action.PROMOTE,
+                NAME: parsed[0],
+                STAGE: parsed[1],
+                NUMBER: int(parsed[2]),
+            }
+    if raise_on_fail:
+        assert_name_is_valid(parsed[0])
+        assert_name_is_valid(parsed[1])
+        if not parsed[2].isdigit():
+            raise InvalidTagName(name)
+    return {}
+
+
+def parse_name(name: str, raise_on_fail: bool = True):
+
+    if ActionSign[Action.REGISTER] in name:
+        return _parse_register(name, raise_on_fail=raise_on_fail)
 
     if ActionSign[Action.PROMOTE] in name:
-        name, stage = name.split(ActionSign[Action.PROMOTE])
-        stage, number = stage.split("-")
-        return {
-            ACTION: Action.PROMOTE,
-            NAME: name,
-            STAGE: stage,
-            NUMBER: int(number),
-        }
+        return _parse_promote(name, raise_on_fail=raise_on_fail)
+
     if raise_on_fail:
-        raise ValueError(f"Unknown tag name: {name}")
+        raise InvalidTagName(name)
     return {}
 
 
+class NAME_REFERENCE(Enum):
+    TAG = TAG
+    NAME = NAME
+
+
+def parse_name_reference(name: str):
+    parsed = parse_name(name, raise_on_fail=False)
+    if not parsed:
+        return NAME_REFERENCE.NAME, name
+    return NAME_REFERENCE.TAG, parsed
+
+
 class Tag(BaseModel):
     action: Action
     name: str
     version: Optional[str]
     stage: Optional[str]
-    creation_date: datetime
+    created_at: datetime
     tag: git.Tag
 
     class Config:
@@ -85,7 +148,7 @@ class Tag(BaseModel):
 def parse_tag(tag: git.Tag):
     return Tag(
         tag=tag,
-        creation_date=datetime.fromtimestamp(tag.tag.tagged_date),
+        created_at=datetime.fromtimestamp(tag.tag.tagged_date),
         **parse_name(tag.name),
     )
 
@@ -123,7 +186,8 @@ def find(
 def create_tag(repo, name, ref, message):
     if all(c.hexsha != ref for c in repo.iter_commits()):
         raise RefNotFound(ref=ref)
-
+    if name in repo.refs:
+        raise TagExists(name=name)
     repo.create_tag(
         name,
         ref=ref,
@@ -131,43 +195,60 @@ def create_tag(repo, name, ref, message):
     )
 
 
-def version_from_tag(artifact: Artifact, tag: git.Tag) -> BaseVersion:
+def version_from_tag(tag: git.Tag) -> BaseVersion:
     mtag = parse_tag(tag)
     return BaseVersion(
-        artifact=artifact,
+        artifact=mtag.name,
         name=mtag.version,
-        creation_date=mtag.creation_date,
+        created_at=mtag.created_at,
         author=tag.tag.tagger.name,
         commit_hexsha=tag.commit.hexsha,
+        tag=tag.name,
     )
 
 
-def promotion_from_tag(artifact: BaseArtifact, tag: git.Tag) -> BasePromotion:
+def promotion_from_tag(
+    artifact: BaseArtifact, tag: git.Tag, version_required: bool
+) -> BasePromotion:
     mtag = parse_tag(tag)
-    version = artifact.find_version(
-        commit_hexsha=tag.commit.hexsha, raise_if_not_found=True
-    )
+    if version_required:
+        version = artifact.find_version(
+            commit_hexsha=tag.commit.hexsha, raise_if_not_found=True
+        ).name  # type: ignore
+    else:
+        version = artifact.find_version(commit_hexsha=tag.commit.hexsha)
+        if version:
+            version = version.name  # type: ignore
+        else:
+            artifact.add_version(
+                BaseVersion(
+                    artifact=mtag.name,
+                    name=tag.commit.hexsha,
+                    created_at=mtag.created_at,
+                    author=tag.tag.tagger.name,
+                    commit_hexsha=tag.commit.hexsha,
+                )
+            )
+            version = tag.commit.hexsha
     return BasePromotion(
-        artifact=artifact.commits[tag.commit.hexsha],
-        version=version.name,  # type: ignore
+        artifact=mtag.name,
+        version=version,
         stage=mtag.stage,
-        creation_date=mtag.creation_date,
+        created_at=mtag.created_at,
         author=tag.tag.tagger.name,
         commit_hexsha=tag.commit.hexsha,
+        tag=tag.name,
     )
 
 
-def index_tag(artifact: BaseArtifact, tag: git.Tag) -> BaseArtifact:
+def index_tag(
+    artifact: BaseArtifact, tag: git.Tag, version_required: bool
+) -> BaseArtifact:
     mtag = parse_tag(tag)
-    hexsha = mtag.tag.commit.hexsha
-    if hexsha not in artifact.commits:
-        # issue a warning that we're ignoring a tag,
-        # because artifact wasn't registered in that commit?
-        return artifact
     if mtag.action == Action.REGISTER:
-        artifact.versions.append(version_from_tag(artifact.commits[hexsha], tag))
+        artifact.add_version(version_from_tag(tag))
     if mtag.action == Action.PROMOTE:
-        artifact.add_promotion(promotion_from_tag(artifact, tag))
+        artifact.add_promotion(promotion_from_tag(artifact, tag, version_required))
     return artifact
 
 
@@ -177,7 +258,13 @@ class TagManager(BaseManager):  # pylint: disable=abstract-method
         # this is important to check that history is not broken
         tags = [parse_tag(t) for t in find(repo=self.repo, action=self.actions)]
         for tag in tags:
-            state.artifacts[tag.name] = index_tag(state.artifacts[tag.name], tag.tag)
+            state.update_artifact(
+                index_tag(
+                    state.find_artifact(tag.name, create_new=True),
+                    tag.tag,
+                    version_required=False,
+                )
+            )
         return state
 
 
@@ -204,8 +291,8 @@ class TagVersionManager(TagManager):
             return {}
         return {
             name: version
-            for name in state.artifacts
-            for version in state.artifacts[name].versions
+            for name, artifact in state.get_artifacts().items()
+            for version in artifact.versions
             if name == art_name and version.name == version_name
         }
 
@@ -213,10 +300,10 @@ class TagVersionManager(TagManager):
 class TagStageManager(TagManager):
     actions: FrozenSet[Action] = frozenset((Action.PROMOTE,))
 
-    def promote(self, name, stage, ref, message):
+    def promote(self, name, stage, ref, message, simple):
         create_tag(
             self.repo,
-            name_tag(Action.PROMOTE, name, stage=stage, repo=self.repo),
+            name_tag(Action.PROMOTE, name, stage=stage, repo=self.repo, simple=simple),
             ref=ref,
             message=message,
         )
@@ -233,9 +320,9 @@ class TagStageManager(TagManager):
             return {}
         return {
             name: promotion
-            for name in state.artifacts
-            for promotion in state.artifacts[name].stages
+            for name, artifact in state.get_artifacts().items()
+            for promotion in artifact.stages
             if name == art_name
             and promotion.commit_hexsha == tag.commit.hexsha
-            and promotion.creation_date == datetime.fromtimestamp(tag.tag.tagged_date)
+            and promotion.created_at == datetime.fromtimestamp(tag.tag.tagged_date)
         }
