@@ -1,10 +1,8 @@
 from functools import total_ordering
-from typing import Union
 
 import semver
 
-from gto.constants import VersionPart
-from gto.exceptions import GTOException, IncomparableVersions, InvalidVersion
+from gto.exceptions import IncomparableVersions, InvalidVersion, WrongArgs
 
 
 class AbstractVersion:
@@ -12,7 +10,7 @@ class AbstractVersion:
 
     def __init__(self, version) -> None:
         if not self.is_valid(version):
-            raise InvalidVersion(version=version, cls=self.__class__)
+            raise InvalidVersion(f"Supplied version '{version}' cannot be parsed")
         self.version = version
 
     @classmethod
@@ -25,50 +23,11 @@ class AbstractVersion:
     def __lt__(self, other):
         raise NotImplementedError
 
-    def bump(self, part: Union[VersionPart, str]) -> "AbstractVersion":
-        raise NotImplementedError
-
     @classmethod
     def get_minimal(cls):
         raise NotImplementedError
 
 
-@total_ordering
-class NumberedVersion(AbstractVersion):
-    @classmethod
-    def is_valid(cls, version):
-        if not isinstance(version, str):
-            return False
-        return version.startswith("v") and version[1:].isdigit()
-
-    def to_number(self):
-        return int(self.version[1:])
-
-    def __eq__(self, other):
-        if isinstance(other, str):
-            other = self.__class__(other)
-        if not isinstance(other, self.__class__):
-            raise IncomparableVersions(self, other)
-        return self.version == other.version
-
-    def __lt__(self, other):
-        if isinstance(other, str):
-            other = self.__class__(other)
-        if not isinstance(other, self.__class__):
-            raise IncomparableVersions(self, other)
-        return self.to_number() < other.to_number()
-
-    def bump(self, part: Union[VersionPart, str] = VersionPart.MAJOR):
-        part = VersionPart(part)
-        if part != VersionPart.MAJOR:
-            raise GTOException(f"With {self.__class__} you can only bump MAJOR part")
-        return self.__class__(f"v{self.to_number() + 1}")
-
-    @classmethod
-    def get_minimal(cls):
-        return cls("v1")
-
-
 @total_ordering
 class SemVer(AbstractVersion):
     """
@@ -80,7 +39,7 @@ class SemVer(AbstractVersion):
         try:
             cls.parse(version)
             return True
-        except ValueError:
+        except (InvalidVersion, ValueError, IndexError) as _:
             return False
 
     @classmethod
@@ -93,9 +52,9 @@ class SemVer(AbstractVersion):
         :return: a new instance
         """
         if not isinstance(version, str):
-            raise ValueError("Version should be of type str")
+            raise InvalidVersion("Version should be of type str")
         if version[0] not in ("v"):
-            raise ValueError(
+            raise InvalidVersion(
                 f"{version}: not a valid semantic version tag. Must start with 'v'"
             )
         return semver.VersionInfo.parse(version[1:])
@@ -114,10 +73,28 @@ class SemVer(AbstractVersion):
             raise IncomparableVersions(self, other)
         return self.parse(self.version) < self.parse(other.version)
 
-    def bump(self, part: Union[VersionPart, str] = VersionPart.PATCH):
-        part = VersionPart(part)
-        next_version = getattr(self.parse(self.version), f"bump_{part.value}")()
-        return self.__class__(f"v{next_version}")
+    def bump_major(self):
+        return self.__class__(f"v{self.parse(self.version).bump_major()}")
+
+    def bump_minor(self):
+        return self.__class__(f"v{self.parse(self.version).bump_minor()}")
+
+    def bump_patch(self):
+        return self.__class__(f"v{self.parse(self.version).bump_patch()}")
+
+    def bump(self, bump_major=False, bump_minor=False, bump_patch=False):
+        if sum(bool(i) for i in (bump_major, bump_minor, bump_patch)) != 1:
+            raise WrongArgs("Need to specify exactly one bump argument")
+        if bump_major:
+            return self.bump_major()
+        if bump_minor:
+            return self.bump_minor()
+        if bump_patch:
+            return self.bump_patch()
+        # TODO: stop using WrongArgs everywhere :)
+        raise WrongArgs(
+            "At least one of bump_major, bump_minor, bump_patch must be True"
+        )
 
     @classmethod
     def get_minimal(cls):
